"""
The groups and report names are defined.
The enums are generated automatically from github markdown file, but they will not be covered in the future. The index
values can be guaranteed too be consistent, and can be saved in database.
"""

from dataclasses import dataclass
from enum import IntEnum, Enum


class ReportTypeGroup(IntEnum):
    inventory_reports = 1
    order_reports = 2
    order_tracking_reports = 3
    pending_order_reports = 4
    returns_reports = 5
    performance_reports = 6
    settlement_reports = 7
    fulfillment_by_amazon_fba_reports = 8
    tax_reports = 9
    browse_tree_report = 10
    easy_ship_reports = 11
    amazon_business_reports = 12
    amazon_pay_report = 13
    b2b_product_opportunities_reports = 14

    @property
    def reports(self):
        return (report for report in ReportType if report.group == self)


@dataclass
class __ReportTypeDefinition:
    group_index: int
    report_index: int
    upload_name: str

    @property
    def index(self):
        return self.group_index * 100 + self.report_index

    @property
    def group(self) -> ReportTypeGroup:
        return ReportTypeGroup(self.group_index)


class ReportType(__ReportTypeDefinition, Enum):
    inventory_report = 1, 1, "GET_FLAT_FILE_OPEN_LISTINGS_DATA"
    all_listings_report = 1, 2, "GET_MERCHANT_LISTINGS_ALL_DATA"
    active_listings_report = 1, 3, "GET_MERCHANT_LISTINGS_DATA"
    inactive_listings_report = 1, 4, "GET_MERCHANT_LISTINGS_INACTIVE_DATA"
    open_listings_report = 1, 5, "GET_MERCHANT_LISTINGS_DATA_BACK_COMPAT"
    open_listings_report_lite = 1, 6, "GET_MERCHANT_LISTINGS_DATA_LITE"
    open_listings_report_liter = 1, 7, "GET_MERCHANT_LISTINGS_DATA_LITER"
    canceled_listings_report = 1, 8, "GET_MERCHANT_CANCELLED_LISTINGS_DATA"
    listing_quality_and_suppressed_listing_report = 1, 9, "GET_MERCHANT_LISTINGS_DEFECT_DATA"
    global_expansion_opportunities_report = 1, 10, "GET_FLAT_FILE_GEO_OPPORTUNITIES"
    referral_fee_preview_report = 1, 11, "GET_REFERRAL_FEE_PREVIEW_REPORT"
    unshipped_orders_report = 2, 1, "GET_FLAT_FILE_ACTIONABLE_ORDER_DATA_SHIPPING"
    requested_or_scheduled_flat_file_order_report_shipping = 2, 2, "GET_FLAT_FILE_ORDER_REPORT_DATA_SHIPPING"
    flat_file_orders_by_last_update_report = 3, 1, "GET_FLAT_FILE_ALL_ORDERS_DATA_BY_LAST_UPDATE_GENERAL"
    flat_file_orders_by_order_date_report = 3, 2, "GET_FLAT_FILE_ALL_ORDERS_DATA_BY_ORDER_DATE_GENERAL"
    flat_file_archived_orders_report = 3, 3, "GET_FLAT_FILE_ARCHIVED_ORDERS_DATA_BY_ORDER_DATE"
    xml_orders_by_last_update_report = 3, 4, "GET_XML_ALL_ORDERS_DATA_BY_LAST_UPDATE_GENERAL"
    xml_orders_by_order_date_report = 3, 5, "GET_XML_ALL_ORDERS_DATA_BY_ORDER_DATE_GENERAL"
    flat_file_pending_orders_report = 4, 1, "GET_FLAT_FILE_PENDING_ORDERS_DATA"
    xml_pending_orders_report = 4, 2, "GET_PENDING_ORDERS_DATA"
    converged_flat_file_pending_orders_report = 4, 3, "GET_CONVERGED_FLAT_FILE_PENDING_ORDERS_DATA"
    xml_returns_report_by_return_date = 5, 1, "GET_XML_RETURNS_DATA_BY_RETURN_DATE"
    flat_file_returns_report_by_return_date = 5, 2, "GET_FLAT_FILE_RETURNS_DATA_BY_RETURN_DATE"
    xml_prime_returns_report_by_return_date = 5, 3, "GET_XML_MFN_PRIME_RETURNS_REPORT"
    csv_prime_returns_report_by_return_date = 5, 4, "GET_CSV_MFN_PRIME_RETURNS_REPORT"
    xml_return_attributes_report_by_return_date = 5, 5, "GET_XML_MFN_SKU_RETURN_ATTRIBUTES_REPORT"
    flat_file_return_attributes_report_by_return_date = 5, 6, "GET_FLAT_FILE_MFN_SKU_RETURN_ATTRIBUTES_REPORT"
    flat_file_feedback_report = 6, 1, "GET_SELLER_FEEDBACK_DATA"
    xml_customer_metrics_report = 6, 2, "GET_V1_SELLER_PERFORMANCE_REPORT"
    seller_performance_report = 6, 3, "GET_V2_SELLER_PERFORMANCE_REPORT"
    flat_file_settlement_report = 7, 1, "GET_V2_SETTLEMENT_REPORT_DATA_FLAT_FILE"
    xml_settlement_report = 7, 2, "GET_V2_SETTLEMENT_REPORT_DATA_XML"
    flat_file_v2_settlement_report = 7, 3, "GET_V2_SETTLEMENT_REPORT_DATA_FLAT_FILE_V2"
    fba_amazon_fulfilled_shipments_report = 8, 1, "GET_AMAZON_FULFILLED_SHIPMENTS_DATA_GENERAL"
    fba_amazon_fulfilled_shipments_report_invoicing = 8, 2, "GET_AMAZON_FULFILLED_SHIPMENTS_DATA_INVOICING"
    fba_amazon_fulfilled_shipments_report_tax = 8, 3, "GET_AMAZON_FULFILLED_SHIPMENTS_DATA_TAX"
    flat_file_all_orders_report_by_last_update = 8, 4, "GET_FLAT_FILE_ALL_ORDERS_DATA_BY_LAST_UPDATE_GENERAL"
    flat_file_all_orders_report_by_order_date = 8, 5, "GET_FLAT_FILE_ALL_ORDERS_DATA_BY_ORDER_DATE_GENERAL"
    xml_all_orders_report_by_last_update = 8, 6, "GET_XML_ALL_ORDERS_DATA_BY_LAST_UPDATE_GENERAL"
    xml_all_orders_report_by_order_date = 8, 7, "GET_XML_ALL_ORDERS_DATA_BY_ORDER_DATE_GENERAL"
    fba_customer_shipment_sales_report = 8, 8, "GET_FBA_FULFILLMENT_CUSTOMER_SHIPMENT_SALES_DATA"
    fba_promotions_report = 8, 9, "GET_FBA_FULFILLMENT_CUSTOMER_SHIPMENT_PROMOTION_DATA"
    fba_customer_taxes = 8, 10, "GET_FBA_FULFILLMENT_CUSTOMER_TAXES_DATA"
    remote_fulfillment_eligibility = 8, 11, "GET_REMOTE_FULFILLMENT_ELIGIBILITY"
    sales_tax_report = 9, 1, "GET_FLAT_FILE_SALES_TAX_DATA"
    amazon_vat_calculation_report = 9, 2, "SC_VAT_TAX_REPORT"
    amazon_vat_transactions_report = 9, 3, "GET_VAT_TRANSACTION_DATA"
    on_demand_gst_merchant_tax_report_b2b = 9, 4, "GET_GST_MTR_B2B_CUSTOM"
    on_demand_gst_merchant_tax_report_b2c = 9, 5, "GET_GST_MTR_B2C_CUSTOM"
    browse_tree_report = 10, 1, "GET_XML_BROWSE_TREE_DATA"
    easyship_report = 11, 1, "GET_EASYSHIP_DOCUMENTS"
    easyship_picked_up_report = 11, 2, "GET_EASYSHIP_PICKEDUP"
    easyship_waiting_for_pick_up_report = 11, 3, "GET_EASYSHIP_WAITING_FOR_PICKUP"
    manage_quotes_report = 12, 1, "RFQD_BULK_DOWNLOAD"
    referral_fee_discounts_report = 12, 2, "FEE_DISCOUNTS_REPORT"
    amazonpay_sandbox_settlement_report = 13, 1, "GET_FLAT_FILE_OFFAMAZONPAYMENTS_SANDBOX_SETTLEMENT_DATA"
    b2b_product_opportunities_recommended_for_you_report = 14, 1, "GET_B2B_PRODUCT_OPPORTUNITIES_RECOMMENDED_FOR_YOU"
    b2b_product_opportunities_not_yet_on_amazon = 14, 2, "GET_B2B_PRODUCT_OPPORTUNITIES_NOT_YET_ON_AMAZON"



import hashlib
import hmac
import json
import os
import urllib
from collections import OrderedDict
from datetime import datetime
from time import sleep
from typing import Union
from urllib.parse import urlparse

import boto3
import requests
from cachetools import TTLCache
from requests import Response
from requests.api import request
from requests.auth import AuthBase

from .cache import RequestCache

TRUE_VALUES = (True, 'true', 't', 'yes', 'y')
FALSE_VALUES = (False, 'false', 'f', 'no', 'n')


def convert_bool(value):
    if isinstance(value, str):
        value = value.lower()
    if value in TRUE_VALUES:
        return True
    elif value in FALSE_VALUES:
        return False
    else:
        raise ValueError('Wrong boolean value')


class AWSSigV4(AuthBase):

    @staticmethod
    def sign_msg(key, msg):
        return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

    def __init__(self, service, **kwargs):
        # Set Service
        self.service = service

        # First, get credentials passed explicitly
        self.aws_access_key_id = kwargs.get('aws_access_key_id')
        self.aws_secret_access_key = kwargs.get('aws_secret_access_key')
        self.aws_session_token = kwargs.get('aws_session_token')
        # Next, try environment variables or use boto3

        # Last, fail if still not found
        if self.aws_access_key_id is None or self.aws_secret_access_key is None:
            raise KeyError("AWS Access Key ID and Secret Access Key are required")

        # Get Region passed explicitly
        self.region = kwargs.get('region')
        # Next, try environment variables or use boto3
        if self.region is None:
            self.region = os.environ.get('SP_AWS_REGION', 'us-east-1')

    def __call__(self, r):
        # Create a date for headers and the credential string
        t = datetime.utcnow()
        self.amzdate = t.strftime('%Y%m%dT%H%M%SZ')
        self.datestamp = t.strftime('%Y%m%d')

        # Parse request to get URL parts
        p = urlparse(r.url)

        host = p.hostname
        uri = urllib.parse.quote(p.path)

        if len(p.query) > 0:
            qs = dict(map(lambda i: i.split('='), p.query.split('&')))
        else:
            qs = dict()

        canonical_querystring = "&".join(map(lambda x: '='.join(x), sorted(qs.items())))
        headers_to_sign = {'host': host, 'x-amz-date': self.amzdate}
        if self.aws_session_token is not None:
            headers_to_sign['x-amz-security-token'] = self.aws_session_token

        ordered_headers = OrderedDict(sorted(headers_to_sign.items(), key=lambda t: t[0]))
        canonical_headers = ''.join(map(lambda h: ":".join(h) + '\n', ordered_headers.items()))
        signed_headers = ';'.join(ordered_headers.keys())

        if r.method == 'GET':
            payload_hash = hashlib.sha256(''.encode('utf-8')).hexdigest()
        else:
            if r.body:
                if isinstance(r.body, str):
                    payload_hash = hashlib.sha256(r.body.encode('utf-8')).hexdigest()
                elif isinstance(r.body, bytes):
                    payload_hash = hashlib.sha256(r.body).hexdigest()
                else:
                    raise TypeError('r.body type should be str or bytes')

            else:
                payload_hash = hashlib.sha256(''.encode('utf-8')).hexdigest()

        canonical_request = '\n'.join([r.method, uri, canonical_querystring,
                                       canonical_headers, signed_headers, payload_hash])

        credential_scope = '/'.join([self.datestamp, self.region, self.service, 'aws4_request'])
        string_to_sign = '\n'.join(['AWS4-HMAC-SHA256', self.amzdate,
                                    credential_scope, hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()])

        kDate = self.sign_msg(('AWS4' + self.aws_secret_access_key).encode('utf-8'), self.datestamp)
        kRegion = self.sign_msg(kDate, self.region)
        kService = self.sign_msg(kRegion, self.service)
        kSigning = self.sign_msg(kService, 'aws4_request')
        signature = hmac.new(kSigning, string_to_sign.encode('utf-8'), hashlib.sha256).hexdigest()

        authorization_header = "AWS4-HMAC-SHA256 Credential={}/{}, SignedHeaders={}, Signature={}".format(
            self.aws_access_key_id, credential_scope, signed_headers, signature)
        r.headers.update({
            'host': host,
            'x-amz-date': self.amzdate,
            'Authorization': authorization_header,
            'x-amz-security-token': self.aws_session_token
        })
        return r


class SellingApiException(BaseException):
    pass


class BaseClient:
    _role_cache = TTLCache(maxsize=10, ttl=3000)
    _access_token_cache = TTLCache(maxsize=1000, ttl=3000)
    request_cache: RequestCache = None

    # Sometime the response are required, however, current architecture is not able to return the response.
    # So, the last response is recorded here. Usually, the client will not request multiple requests. So, it will not
    # cause the data conflict.
    last_response: Response = None

    def __init__(self, *,
                 role_arn: str = None,
                 endpoint: str = None,
                 region: str = None,
                 marketplace_id: str = None,
                 refresh_token: str = None,
                 aws_access_key: str = None,
                 aws_secret_key: str = None,
                 lwa_client_key: str = None,
                 lwa_client_secret: str = None,
                 use_cache=False,
                 ):
        """Create a base client.

        Args:
            role_arn: "arn:aws:iam::xxxxxxxxxxxx:role/sp_api_role"
            endpoint: "https://sellingpartnerapi-eu.amazon.com"
            marketplace_id: "A1F83G8C2ARO7P"
            refresh_token:  "Atzr|xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            aws_access_key: 'xxxxxxxxxxxxxxxxxxxx'
            aws_secret_key: "xxxxx/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            use_cache: if True, check if cached and return cached response, or get new response and save
        """
        if role_arn is None:
            role_arn = os.environ.get('SP_API_ROLE_ARN')
        if endpoint is None:
            endpoint = os.environ.get('SP_API_ENDPOINT')
        if region is None:
            region = os.environ.get('SP_API_REGION')
        if marketplace_id is None:
            marketplace_id = os.environ.get('SP_API_MARKETPLACE_ID')
        if refresh_token is None:
            refresh_token = os.environ.get('SP_API_REFRESH_TOKEN')
        if aws_access_key is None:
            aws_access_key = os.environ.get('SP_API_AWS_ACCESS_KEY')
        if aws_secret_key is None:
            aws_secret_key = os.environ.get('SP_API_AWS_SECRET_KEY')
        if lwa_client_key is None:
            lwa_client_key = os.environ.get('SP_API_LWA_CLIENT_KEY')
        if lwa_client_secret is None:
            lwa_client_secret = os.environ.get('SP_API_LWA_CLIENT_SECRET')
        self._client = boto3.client('sts', aws_access_key_id=aws_access_key, aws_secret_access_key=aws_secret_key)
        self._role_arn = role_arn
        self._endpoint = endpoint
        self._region = region
        self._marketplace_id = marketplace_id
        self._refresh_token = refresh_token
        self._aws_access_key = aws_access_key
        self._aws_secret_key = aws_secret_key
        self._client_id = lwa_client_key
        self._client_secret = lwa_client_secret
        self.use_cache = use_cache
        if self.use_cache:
            self.request_cache = RequestCache(
                self._endpoint, self._marketplace_id, self._refresh_token, self._aws_access_key)

    def __get_access_token(self, refresh_token):
        if refresh_token not in self._access_token_cache:
            data = {
                'grant_type': 'refresh_token',
                'client_id': self._client_id,
                'client_secret': self._client_secret,
                'refresh_token': refresh_token,
            }
            headers = {
                'User-Agent': 'python-sp-api',
                'content-type': 'application/x-www-form-urlencoded;charset=UTF-8',
            }
            response = requests.post('https://api.amazon.com/auth/o2/token', data=data, headers=headers)
            access_token = response.json()['access_token']
            self._access_token_cache[refresh_token] = access_token
        return self._access_token_cache[refresh_token]

    @staticmethod
    def _get_response_json(response: requests.Response):
        try:
            return response.json()
        except json.JSONDecodeError:
            try:
                import demjson
            except ImportError:
                raise SellingApiException('Could not parse response, please try to install demjson')
            return demjson.decode(response.text)

    def request(self, path: str, *, data: Union[dict, bytes] = None, params: dict = None, headers=None,
                method='GET', check_exception=True) -> Response:

        # process params
        parsed_params = {}
        params is None or parsed_params.update(params)

        # process data
        parsed_data: bytes
        if data is None:
            parsed_data = b''
        elif isinstance(data, dict):
            parsed_data = json.dumps(data).encode()
        elif isinstance(data, bytes):
            parsed_data = data
        else:
            raise TypeError('data should be a dict or bytes')

        # process cache, will be removed in next main version
        if self.use_cache:
            cache_keys: RequestCache.KEYS_TYPE = (path, method, params, data, headers)
            response = self.request_cache[cache_keys]
            if response is not None:
                return response

        # process headers
        parsed_headers = {
            'host': self._endpoint[8:],
            'user-agent': 'python-sp-api',
            'x-amz-access-token': self.__get_access_token(self._refresh_token),
            'x-amz-date': datetime.utcnow().strftime('%Y%m%dT%H%M%SZ'),
            'content-type': 'application/json'
        }
        headers is None or parsed_headers.update(headers)

        # process auth
        if 'role' not in self._role_cache:
            role = self._client.assume_role(RoleArn=self._role_arn, RoleSessionName='guid').get('Credentials')
            self._role_cache['role'] = role
        role = self._role_cache['role']
        auth = AWSSigV4('execute-api',
                        aws_access_key_id=role.get('AccessKeyId'),
                        aws_secret_access_key=role.get('SecretAccessKey'),
                        region=self._region,
                        aws_session_token=role.get('SessionToken'))
        url = self._endpoint + path

        # process quota exceeded exception
        while True:
            response = request(method=method, url=url, data=parsed_data, headers=parsed_headers,
                               auth=auth, params=params)

            self.last_response = response
            if not check_exception:
                break

            # If found error, and the error is QuotaExceed, just resend the request
            e = self._get_response_json(response).get('errors', None)
            if e:
                if len(e) == 1 and 'code' in e[0] and e[0]['code'] in ('QuotaExceeded',):
                    sleep(0.1)
                    continue
                else:
                    raise SellingApiException(e)
            else:
                break

        # process cache, will be removed in the next main version
        if self.use_cache:
            cache_keys: RequestCache.KEYS_TYPE = (path, method, params, data, headers)
            self.request_cache[cache_keys] = response

        return response


class BaseDictObject:
    def __init__(self, data):
        self.data: dict = data

    def _get_value(self, klass: type, name: str):
        value = self.data.get(name, None)
        if value is None and klass is dict:
            return None
        return klass(value)


class BaseClients:
    def __init__(self, *,
                 role_arn: str = None,
                 endpoint: str = None,
                 region: str = None,
                 marketplace_id: str = None,
                 refresh_token: str = None,
                 aws_access_key: str = None,
                 aws_secret_key: str = None,
                 lwa_client_key: str = None,
                 lwa_client_secret: str = None,
                 ):
        """Create a base client.

        Args:
            role_arn: "arn:aws:iam::xxxxxxxxxxxx:role/sp_api_role"
            endpoint: "https://sellingpartnerapi-eu.amazon.com"
            marketplace_id: "A1F83G8C2ARO7P"
            refresh_token:  "Atzr|xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" \
                            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            aws_access_key: 'xxxxxxxxxxxxxxxxxxxx'
            aws_secret_key: "xxxxx/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            use_cache: if True, check if cached and return cached response, or get new response and save
        """
        if role_arn is None:
            role_arn = os.environ.get('SP_API_ROLE_ARN')
        if endpoint is None:
            endpoint = os.environ.get('SP_API_ENDPOINT')
        if region is None:
            region = os.environ.get('SP_API_REGION')
        if marketplace_id is None:
            marketplace_id = os.environ.get('SP_API_MARKETPLACE_ID')
        if refresh_token is None:
            refresh_token = os.environ.get('SP_API_REFRESH_TOKEN')
        if aws_access_key is None:
            aws_access_key = os.environ.get('SP_API_AWS_ACCESS_KEY')
        if aws_secret_key is None:
            aws_secret_key = os.environ.get('SP_API_AWS_SECRET_KEY')
        if lwa_client_key is None:
            lwa_client_key = os.environ.get('SP_API_LWA_CLIENT_KEY')
        if lwa_client_secret is None:
            lwa_client_secret = os.environ.get('SP_API_LWA_CLIENT_SECRET')
        self._role_arn = role_arn
        self._endpoint = endpoint
        self._region = region
        self._marketplace_id = marketplace_id
        self._refresh_token = refresh_token
        self._aws_access_key = aws_access_key
        self._aws_secret_key = aws_secret_key
        self._client_id = lwa_client_key
        self._client_secret = lwa_client_secret
        self._parameters = dict(
            role_arn=role_arn,
            endpoint=endpoint,
            region=region,
            marketplace_id=marketplace_id,
            refresh_token=refresh_token,
            aws_access_key=aws_access_key,
            aws_secret_key=aws_secret_key,
            lwa_client_key=lwa_client_key,
            lwa_client_secret=lwa_client_secret,
        )



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class GetTransactionResponse(__BaseDictObject):
    """
    The response schema for the getTransaction operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: TransactionStatus = self._get_value(TransactionStatus, "payload")
        else:
            self.payload: TransactionStatus = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class TransactionStatus(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "transactionStatus" in data:
            self.transactionStatus: Transaction = self._get_value(Transaction, "transactionStatus")
        else:
            self.transactionStatus: Transaction = None


class Transaction(__BaseDictObject):
    """
    The transaction status.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class VendorTransactionStatusV1Client(__BaseClient):
    def getTransaction(
        self,
        transactionId: str,
    ):
        """
                Returns the status of the transaction that you specify.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/vendor/transactions/v1/transactions/{transactionId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetTransactionResponse,
            400: GetTransactionResponse,
            401: GetTransactionResponse,
            403: GetTransactionResponse,
            404: GetTransactionResponse,
            415: GetTransactionResponse,
            429: GetTransactionResponse,
            500: GetTransactionResponse,
            503: GetTransactionResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class RestrictionList(__BaseDictObject):
    """
    A list of restrictions for the specified Amazon catalog item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "restrictions" in data:
            self.restrictions: _List[Restriction] = [Restriction(datum) for datum in data["restrictions"]]
        else:
            self.restrictions: _List[Restriction] = []


class Restriction(__BaseDictObject):
    """
    A listing restriction, optionally qualified by a condition, with a list of reasons for the restriction.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceId" in data:
            self.marketplaceId: str = self._get_value(str, "marketplaceId")
        else:
            self.marketplaceId: str = None
        if "conditionType" in data:
            self.conditionType: str = self._get_value(str, "conditionType")
        else:
            self.conditionType: str = None
        if "reasons" in data:
            self.reasons: _List[Reason] = [Reason(datum) for datum in data["reasons"]]
        else:
            self.reasons: _List[Reason] = []


class Reason(__BaseDictObject):
    """
    A reason for the restriction, including path forward links that may allow Selling Partners to remove the restriction, if available.
    """

    def __init__(self, data):
        super().__init__(data)
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "reasonCode" in data:
            self.reasonCode: str = self._get_value(str, "reasonCode")
        else:
            self.reasonCode: str = None
        if "links" in data:
            self.links: _List[Link] = [Link(datum) for datum in data["links"]]
        else:
            self.links: _List[Link] = []


class Link(__BaseDictObject):
    """
    A link to resources related to a listing restriction.
    """

    def __init__(self, data):
        super().__init__(data)
        if "resource" in data:
            self.resource: str = self._get_value(str, "resource")
        else:
            self.resource: str = None
        if "verb" in data:
            self.verb: str = self._get_value(str, "verb")
        else:
            self.verb: str = None
        if "title" in data:
            self.title: str = self._get_value(str, "title")
        else:
            self.title: str = None
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class ListingsRestrictions20210801Client(__BaseClient):
    def getListingsRestrictions(
        self,
        asin: str,
        sellerId: str,
        marketplaceIds: _List[str],
        conditionType: str = None,
        reasonLocale: str = None,
    ):
        """
                Returns listing restrictions for an item in the Amazon Catalog.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 5 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/listings/2021-08-01/restrictions"
        params = {}
        if asin is not None:
            params["asin"] = asin
        if conditionType is not None:
            params["conditionType"] = conditionType
        if sellerId is not None:
            params["sellerId"] = sellerId
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if reasonLocale is not None:
            params["reasonLocale"] = reasonLocale
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: RestrictionList,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class TransactionStatus(__BaseDictObject):
    """
    The payload for the getTransactionStatus operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionStatus" in data:
            self.transactionStatus: Transaction = self._get_value(Transaction, "transactionStatus")
        else:
            self.transactionStatus: Transaction = None


class Transaction(__BaseDictObject):
    """
    The transaction status details.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class VendorDirectFulfillmentTransactions20211228Client(__BaseClient):
    def getTransactionStatus(
        self,
        transactionId: str,
    ):
        """
                Returns the status of the transaction indicated by the specified transactionId.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 10 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/vendor/directFulfillment/transactions/2021-12-28/transactions/{transactionId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: TransactionStatus,
            400: ErrorList,
            401: Error,
            403: Error,
            404: Error,
            415: Error,
            429: Error,
            500: Error,
            503: Error,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class SchemaLink(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "link" in data:
            self.link: dict = self._get_value(dict, "link")
        else:
            self.link: dict = None
        if "checksum" in data:
            self.checksum: str = self._get_value(str, "checksum")
        else:
            self.checksum: str = None


class ProductTypeDefinition(__BaseDictObject):
    """
    A product type definition represents the attributes and data requirements for a product type in the Amazon catalog. Product type definitions are used interchangeably between the Selling Partner API for Listings Items, Selling Partner API for Catalog Items, and JSON-based listings feeds in the Selling Partner API for Feeds.
    """

    def __init__(self, data):
        super().__init__(data)
        if "metaSchema" in data:
            self.metaSchema: SchemaLink = self._get_value(SchemaLink, "metaSchema")
        else:
            self.metaSchema: SchemaLink = None
        if "schema" in data:
            self.schema: SchemaLink = self._get_value(SchemaLink, "schema")
        else:
            self.schema: SchemaLink = None
        if "requirements" in data:
            self.requirements: str = self._get_value(str, "requirements")
        else:
            self.requirements: str = None
        if "requirementsEnforced" in data:
            self.requirementsEnforced: str = self._get_value(str, "requirementsEnforced")
        else:
            self.requirementsEnforced: str = None
        if "propertyGroups" in data:
            self.propertyGroups: dict = self._get_value(dict, "propertyGroups")
        else:
            self.propertyGroups: dict = None
        if "locale" in data:
            self.locale: str = self._get_value(str, "locale")
        else:
            self.locale: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "productType" in data:
            self.productType: str = self._get_value(str, "productType")
        else:
            self.productType: str = None
        if "productTypeVersion" in data:
            self.productTypeVersion: ProductTypeVersion = self._get_value(ProductTypeVersion, "productTypeVersion")
        else:
            self.productTypeVersion: ProductTypeVersion = None


class PropertyGroup(__BaseDictObject):
    """
    A property group represents a logical grouping of schema properties that can be used for display or informational purposes.
    """

    def __init__(self, data):
        super().__init__(data)
        if "title" in data:
            self.title: str = self._get_value(str, "title")
        else:
            self.title: str = None
        if "description" in data:
            self.description: str = self._get_value(str, "description")
        else:
            self.description: str = None
        if "propertyNames" in data:
            self.propertyNames: _List[str] = [str(datum) for datum in data["propertyNames"]]
        else:
            self.propertyNames: _List[str] = []


class ProductTypeVersion(__BaseDictObject):
    """
    The version details for an Amazon product type.
    """

    def __init__(self, data):
        super().__init__(data)
        if "version" in data:
            self.version: str = self._get_value(str, "version")
        else:
            self.version: str = None
        if "latest" in data:
            self.latest: bool = self._get_value(convert_bool, "latest")
        else:
            self.latest: bool = None
        if "releaseCandidate" in data:
            self.releaseCandidate: bool = self._get_value(convert_bool, "releaseCandidate")
        else:
            self.releaseCandidate: bool = None


class ProductType(__BaseDictObject):
    """
    An Amazon product type with a definition available.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []


class ProductTypeList(__BaseDictObject):
    """
    A list of Amazon product types with definitions available.
    """

    def __init__(self, data):
        super().__init__(data)
        if "productTypes" in data:
            self.productTypes: _List[ProductType] = [ProductType(datum) for datum in data["productTypes"]]
        else:
            self.productTypes: _List[ProductType] = []


class ProductTypeDefinitions20200901Client(__BaseClient):
    def searchDefinitionsProductTypes(
        self,
        marketplaceIds: _List[str],
        keywords: _List[str] = None,
    ):
        """
                Search for and return a list of Amazon product types that have definitions available.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 5 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/definitions/2020-09-01/productTypes"
        params = {}
        if keywords is not None:
            params["keywords"] = ",".join(map(str, keywords))
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ProductTypeList,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getDefinitionsProductType(
        self,
        productType: str,
        marketplaceIds: _List[str],
        sellerId: str = None,
        productTypeVersion: str = None,
        requirements: str = None,
        requirementsEnforced: str = None,
        locale: str = None,
    ):
        """
                Retrieve an Amazon product type definition.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 5 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/definitions/2020-09-01/productTypes/{productType}"
        params = {}
        if sellerId is not None:
            params["sellerId"] = sellerId
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if productTypeVersion is not None:
            params["productTypeVersion"] = productTypeVersion
        if requirements is not None:
            params["requirements"] = requirements
        if requirementsEnforced is not None:
            params["requirementsEnforced"] = requirementsEnforced
        if locale is not None:
            params["locale"] = locale
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ProductTypeDefinition,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .aplus_content_2020_11_01 import AplusContent20201101Client
from .authorization_v1 import AuthorizationV1Client
from .catalog_items_2020_12_01 import CatalogItems20201201Client
from .catalog_items_v0 import CatalogItemsV0Client
from .fba_inbound_eligibility_v1 import FbaInboundEligibilityV1Client
from .fba_inventory_v1 import FbaInventoryV1Client
from .fba_small_and_light_v1 import FbaSmallAndLightV1Client
from .feeds_2020_09_04 import Feeds20200904Client
from .feeds_2021_06_30 import Feeds20210630Client
from .finances_v0 import FinancesV0Client
from .fulfillment_inbound_v0 import FulfillmentInboundV0Client
from .fulfillment_outbound_2020_07_01 import FulfillmentOutbound20200701Client
from .listings_items_2020_09_01 import ListingsItems20200901Client
from .listings_items_2021_08_01 import ListingsItems20210801Client
from .listings_restrictions_2021_08_01 import ListingsRestrictions20210801Client
from .merchant_fulfillment_v0 import MerchantFulfillmentV0Client
from .messaging_v1 import MessagingV1Client
from .notifications_v1 import NotificationsV1Client
from .orders_v0 import OrdersV0Client
from .product_fees_v0 import ProductFeesV0Client
from .product_pricing_v0 import ProductPricingV0Client
from .product_type_definitions_2020_09_01 import ProductTypeDefinitions20200901Client
from .reports_2020_09_04 import Reports20200904Client
from .reports_2021_06_30 import Reports20210630Client
from .sales_v1 import SalesV1Client
from .sellers_v1 import SellersV1Client
from .services_v1 import ServicesV1Client
from .shipment_invoicing_v0 import ShipmentInvoicingV0Client
from .shipping_v1 import ShippingV1Client
from .solicitations_v1 import SolicitationsV1Client
from .tokens_2021_03_01 import Tokens20210301Client
from .uploads_2020_11_01 import Uploads20201101Client
from .vendor_direct_fulfillment_inventory_v1 import VendorDirectFulfillmentInventoryV1Client
from .vendor_direct_fulfillment_orders_2021_12_28 import VendorDirectFulfillmentOrders20211228Client
from .vendor_direct_fulfillment_orders_v1 import VendorDirectFulfillmentOrdersV1Client
from .vendor_direct_fulfillment_payments_v1 import VendorDirectFulfillmentPaymentsV1Client
from .vendor_direct_fulfillment_sandbox_test_data_2021_10_28 import VendorDirectFulfillmentSandboxTestData20211028Client
from .vendor_direct_fulfillment_shipping_2021_12_28 import VendorDirectFulfillmentShipping20211228Client
from .vendor_direct_fulfillment_shipping_v1 import VendorDirectFulfillmentShippingV1Client
from .vendor_direct_fulfillment_transactions_2021_12_28 import VendorDirectFulfillmentTransactions20211228Client
from .vendor_direct_fulfillment_transactions_v1 import VendorDirectFulfillmentTransactionsV1Client
from .vendor_invoices_v1 import VendorInvoicesV1Client
from .vendor_orders_v1 import VendorOrdersV1Client
from .vendor_shipments_v1 import VendorShipmentsV1Client
from .vendor_transaction_status_v1 import VendorTransactionStatusV1Client
from .marketplaces import MarketPlaces
from .report_types import ReportType, ReportTypeGroup
from .base import BaseClients

try:
    from functools import cached_property
except ImportError:
    try:
        from cached_property import cached_property
    except ImportError:
        raise "Please install cached_property for python < 3.8"


class AmazonSpApiClients(BaseClients):
    @cached_property
    def aplus_content_2020_11_01(self):
        return AplusContent20201101Client(**self._parameters)

    @cached_property
    def authorization_v1(self):
        return AuthorizationV1Client(**self._parameters)

    @cached_property
    def catalog_items_2020_12_01(self):
        return CatalogItems20201201Client(**self._parameters)

    @cached_property
    def catalog_items_v0(self):
        return CatalogItemsV0Client(**self._parameters)

    @cached_property
    def fba_inbound_eligibility_v1(self):
        return FbaInboundEligibilityV1Client(**self._parameters)

    @cached_property
    def fba_inventory_v1(self):
        return FbaInventoryV1Client(**self._parameters)

    @cached_property
    def fba_small_and_light_v1(self):
        return FbaSmallAndLightV1Client(**self._parameters)

    @cached_property
    def feeds_2020_09_04(self):
        return Feeds20200904Client(**self._parameters)

    @cached_property
    def feeds_2021_06_30(self):
        return Feeds20210630Client(**self._parameters)

    @cached_property
    def finances_v0(self):
        return FinancesV0Client(**self._parameters)

    @cached_property
    def fulfillment_inbound_v0(self):
        return FulfillmentInboundV0Client(**self._parameters)

    @cached_property
    def fulfillment_outbound_2020_07_01(self):
        return FulfillmentOutbound20200701Client(**self._parameters)

    @cached_property
    def listings_items_2020_09_01(self):
        return ListingsItems20200901Client(**self._parameters)

    @cached_property
    def listings_items_2021_08_01(self):
        return ListingsItems20210801Client(**self._parameters)

    @cached_property
    def listings_restrictions_2021_08_01(self):
        return ListingsRestrictions20210801Client(**self._parameters)

    @cached_property
    def merchant_fulfillment_v0(self):
        return MerchantFulfillmentV0Client(**self._parameters)

    @cached_property
    def messaging_v1(self):
        return MessagingV1Client(**self._parameters)

    @cached_property
    def notifications_v1(self):
        return NotificationsV1Client(**self._parameters)

    @cached_property
    def orders_v0(self):
        return OrdersV0Client(**self._parameters)

    @cached_property
    def product_fees_v0(self):
        return ProductFeesV0Client(**self._parameters)

    @cached_property
    def product_pricing_v0(self):
        return ProductPricingV0Client(**self._parameters)

    @cached_property
    def product_type_definitions_2020_09_01(self):
        return ProductTypeDefinitions20200901Client(**self._parameters)

    @cached_property
    def reports_2020_09_04(self):
        return Reports20200904Client(**self._parameters)

    @cached_property
    def reports_2021_06_30(self):
        return Reports20210630Client(**self._parameters)

    @cached_property
    def sales_v1(self):
        return SalesV1Client(**self._parameters)

    @cached_property
    def sellers_v1(self):
        return SellersV1Client(**self._parameters)

    @cached_property
    def services_v1(self):
        return ServicesV1Client(**self._parameters)

    @cached_property
    def shipment_invoicing_v0(self):
        return ShipmentInvoicingV0Client(**self._parameters)

    @cached_property
    def shipping_v1(self):
        return ShippingV1Client(**self._parameters)

    @cached_property
    def solicitations_v1(self):
        return SolicitationsV1Client(**self._parameters)

    @cached_property
    def tokens_2021_03_01(self):
        return Tokens20210301Client(**self._parameters)

    @cached_property
    def uploads_2020_11_01(self):
        return Uploads20201101Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_inventory_v1(self):
        return VendorDirectFulfillmentInventoryV1Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_orders_2021_12_28(self):
        return VendorDirectFulfillmentOrders20211228Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_orders_v1(self):
        return VendorDirectFulfillmentOrdersV1Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_payments_v1(self):
        return VendorDirectFulfillmentPaymentsV1Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_sandbox_test_data_2021_10_28(self):
        return VendorDirectFulfillmentSandboxTestData20211028Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_shipping_2021_12_28(self):
        return VendorDirectFulfillmentShipping20211228Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_shipping_v1(self):
        return VendorDirectFulfillmentShippingV1Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_transactions_2021_12_28(self):
        return VendorDirectFulfillmentTransactions20211228Client(**self._parameters)

    @cached_property
    def vendor_direct_fulfillment_transactions_v1(self):
        return VendorDirectFulfillmentTransactionsV1Client(**self._parameters)

    @cached_property
    def vendor_invoices_v1(self):
        return VendorInvoicesV1Client(**self._parameters)

    @cached_property
    def vendor_orders_v1(self):
        return VendorOrdersV1Client(**self._parameters)

    @cached_property
    def vendor_shipments_v1(self):
        return VendorShipmentsV1Client(**self._parameters)

    @cached_property
    def vendor_transaction_status_v1(self):
        return VendorTransactionStatusV1Client(**self._parameters)


version = "1.7.10"
name = "amazon-sp-api-clients"
author = "Haoyu Pan"
author_email = "panhaoyu.china@outlook.com"
description = "Amazon selling partner api clients."
url = "https://github.com/panhaoyu/sp-api-clients"



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class Subscription(__BaseDictObject):
    """
    Represents a subscription to receive notifications.
    """

    def __init__(self, data):
        super().__init__(data)
        if "subscriptionId" in data:
            self.subscriptionId: str = self._get_value(str, "subscriptionId")
        else:
            self.subscriptionId: str = None
        if "payloadVersion" in data:
            self.payloadVersion: str = self._get_value(str, "payloadVersion")
        else:
            self.payloadVersion: str = None
        if "destinationId" in data:
            self.destinationId: str = self._get_value(str, "destinationId")
        else:
            self.destinationId: str = None


class CreateSubscriptionResponse(__BaseDictObject):
    """
    The response schema for the createSubscription operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Subscription = self._get_value(Subscription, "payload")
        else:
            self.payload: Subscription = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class CreateSubscriptionRequest(__BaseDictObject):
    """
    The request schema for the createSubscription operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payloadVersion" in data:
            self.payloadVersion: str = self._get_value(str, "payloadVersion")
        else:
            self.payloadVersion: str = None
        if "destinationId" in data:
            self.destinationId: str = self._get_value(str, "destinationId")
        else:
            self.destinationId: str = None


class GetSubscriptionByIdResponse(__BaseDictObject):
    """
    The response schema for the getSubscriptionById operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Subscription = self._get_value(Subscription, "payload")
        else:
            self.payload: Subscription = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetSubscriptionResponse(__BaseDictObject):
    """
    The response schema for the getSubscription operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Subscription = self._get_value(Subscription, "payload")
        else:
            self.payload: Subscription = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class DeleteSubscriptionByIdResponse(__BaseDictObject):
    """
    The response schema for the deleteSubscriptionById operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Destination(__BaseDictObject):
    """
    Represents a destination created when you call the createDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "destinationId" in data:
            self.destinationId: str = self._get_value(str, "destinationId")
        else:
            self.destinationId: str = None
        if "resource" in data:
            self.resource: DestinationResource = self._get_value(DestinationResource, "resource")
        else:
            self.resource: DestinationResource = None


class DestinationResource(__BaseDictObject):
    """
    The destination resource types.
    """

    def __init__(self, data):
        super().__init__(data)
        if "sqs" in data:
            self.sqs: SqsResource = self._get_value(SqsResource, "sqs")
        else:
            self.sqs: SqsResource = None
        if "eventBridge" in data:
            self.eventBridge: EventBridgeResource = self._get_value(EventBridgeResource, "eventBridge")
        else:
            self.eventBridge: EventBridgeResource = None


class DestinationResourceSpecification(__BaseDictObject):
    """
    The information required to create a destination resource. Applications should use one resource type (sqs or eventBridge) per destination.
    """

    def __init__(self, data):
        super().__init__(data)
        if "sqs" in data:
            self.sqs: SqsResource = self._get_value(SqsResource, "sqs")
        else:
            self.sqs: SqsResource = None
        if "eventBridge" in data:
            self.eventBridge: EventBridgeResourceSpecification = self._get_value(
                EventBridgeResourceSpecification, "eventBridge"
            )
        else:
            self.eventBridge: EventBridgeResourceSpecification = None


class SqsResource(__BaseDictObject):
    """
    The information required to create an Amazon Simple Queue Service (Amazon SQS) queue destination.
    """

    def __init__(self, data):
        super().__init__(data)
        if "arn" in data:
            self.arn: str = self._get_value(str, "arn")
        else:
            self.arn: str = None


class EventBridgeResourceSpecification(__BaseDictObject):
    """
    The information required to create an Amazon EventBridge destination.
    """

    def __init__(self, data):
        super().__init__(data)
        if "region" in data:
            self.region: str = self._get_value(str, "region")
        else:
            self.region: str = None
        if "accountId" in data:
            self.accountId: str = self._get_value(str, "accountId")
        else:
            self.accountId: str = None


class EventBridgeResource(__BaseDictObject):
    """
    Represents an Amazon EventBridge destination.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "region" in data:
            self.region: str = self._get_value(str, "region")
        else:
            self.region: str = None
        if "accountId" in data:
            self.accountId: str = self._get_value(str, "accountId")
        else:
            self.accountId: str = None


class CreateDestinationRequest(__BaseDictObject):
    """
    The request schema for the createDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "resourceSpecification" in data:
            self.resourceSpecification: DestinationResourceSpecification = self._get_value(
                DestinationResourceSpecification, "resourceSpecification"
            )
        else:
            self.resourceSpecification: DestinationResourceSpecification = None
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None


class CreateDestinationResponse(__BaseDictObject):
    """
    The response schema for the createDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Destination = self._get_value(Destination, "payload")
        else:
            self.payload: Destination = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetDestinationResponse(__BaseDictObject):
    """
    The response schema for the getDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Destination = self._get_value(Destination, "payload")
        else:
            self.payload: Destination = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetDestinationsResponse(__BaseDictObject):
    """
    The response schema for the getDestinations operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: DestinationList = self._get_value(DestinationList, "payload")
        else:
            self.payload: DestinationList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class DeleteDestinationResponse(__BaseDictObject):
    """
    The response schema for the deleteDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class DestinationList(list, _List["Destination"]):
    """
    A list of destinations.
    """

    def __init__(self, data):
        super().__init__([Destination(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class NotificationType(str):
    """
       The type of notification.
    For more information about notification types, see [the Notifications API Use Case Guide](doc:notifications-api-v1-use-case-guide).
    """


class NotificationsV1Client(__BaseClient):
    def getSubscription(
        self,
        notificationType: str,
    ):
        """
                Returns information about subscriptions of the specified notification type. You can use this API to get subscription information when you do not have a subscription identifier.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/subscriptions/{notificationType}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetSubscriptionResponse,
            400: GetSubscriptionResponse,
            403: GetSubscriptionResponse,
            404: GetSubscriptionResponse,
            413: GetSubscriptionResponse,
            415: GetSubscriptionResponse,
            429: GetSubscriptionResponse,
            500: GetSubscriptionResponse,
            503: GetSubscriptionResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createSubscription(
        self,
        data: CreateSubscriptionRequest,
        notificationType: str,
    ):
        """
                Creates a subscription for the specified notification type to be delivered to the specified destination. Before you can subscribe, you must first create the destination by calling the createDestination operation.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/subscriptions/{notificationType}"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: CreateSubscriptionResponse,
            400: CreateSubscriptionResponse,
            403: CreateSubscriptionResponse,
            404: CreateSubscriptionResponse,
            409: CreateSubscriptionResponse,
            413: CreateSubscriptionResponse,
            415: CreateSubscriptionResponse,
            429: CreateSubscriptionResponse,
            500: CreateSubscriptionResponse,
            503: CreateSubscriptionResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getSubscriptionById(
        self,
        subscriptionId: str,
        notificationType: str,
    ):
        """
                Returns information about a subscription for the specified notification type. The getSubscriptionById API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/subscriptions/{notificationType}/{subscriptionId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetSubscriptionByIdResponse,
            400: GetSubscriptionByIdResponse,
            403: GetSubscriptionByIdResponse,
            404: GetSubscriptionResponse,
            409: GetSubscriptionByIdResponse,
            413: GetSubscriptionByIdResponse,
            415: GetSubscriptionByIdResponse,
            429: GetSubscriptionByIdResponse,
            500: GetSubscriptionByIdResponse,
            503: GetSubscriptionByIdResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def deleteSubscriptionById(
        self,
        subscriptionId: str,
        notificationType: str,
    ):
        """
                Deletes the subscription indicated by the subscription identifier and notification type that you specify. The subscription identifier can be for any subscription associated with your application. After you successfully call this operation, notifications will stop being sent for the associated subscription. The deleteSubscriptionById API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/subscriptions/{notificationType}/{subscriptionId}"
        params = {}
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: DeleteSubscriptionByIdResponse,
            400: DeleteSubscriptionByIdResponse,
            403: DeleteSubscriptionByIdResponse,
            404: DeleteSubscriptionByIdResponse,
            409: DeleteSubscriptionByIdResponse,
            413: DeleteSubscriptionByIdResponse,
            415: DeleteSubscriptionByIdResponse,
            429: DeleteSubscriptionByIdResponse,
            500: DeleteSubscriptionByIdResponse,
            503: DeleteSubscriptionByIdResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getDestinations(
        self,
    ):
        """
                Returns information about all destinations. The getDestinations API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/destinations"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetDestinationsResponse,
            400: GetDestinationsResponse,
            403: GetDestinationsResponse,
            404: GetDestinationsResponse,
            409: GetDestinationsResponse,
            413: GetDestinationsResponse,
            415: GetDestinationsResponse,
            429: GetDestinationsResponse,
            500: GetDestinationsResponse,
            503: GetDestinationsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createDestination(
        self,
        data: CreateDestinationRequest,
    ):
        """
                Creates a destination resource to receive notifications. The createDestination API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/destinations"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: CreateDestinationResponse,
            400: CreateDestinationResponse,
            403: CreateDestinationResponse,
            404: CreateDestinationResponse,
            409: CreateDestinationResponse,
            413: CreateDestinationResponse,
            415: CreateDestinationResponse,
            429: CreateDestinationResponse,
            500: CreateDestinationResponse,
            503: CreateDestinationResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getDestination(
        self,
        destinationId: str,
    ):
        """
                Returns information about the destination that you specify. The getDestination API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/destinations/{destinationId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetDestinationResponse,
            400: GetDestinationResponse,
            403: GetDestinationResponse,
            404: GetDestinationResponse,
            409: GetDestinationResponse,
            413: GetDestinationResponse,
            415: GetDestinationResponse,
            429: GetDestinationResponse,
            500: GetDestinationResponse,
            503: GetDestinationResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def deleteDestination(
        self,
        destinationId: str,
    ):
        """
                Deletes the destination that you specify. The deleteDestination API is grantless. For more information, see "Grantless operations" in the Selling Partner API Developer Guide.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/notifications/v1/destinations/{destinationId}"
        params = {}
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: DeleteDestinationResponse,
            400: DeleteDestinationResponse,
            403: DeleteDestinationResponse,
            404: DeleteDestinationResponse,
            409: DeleteDestinationResponse,
            413: DeleteDestinationResponse,
            415: DeleteDestinationResponse,
            429: DeleteDestinationResponse,
            500: DeleteDestinationResponse,
            503: DeleteDestinationResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



import setuptools

import amazon_sp_api_clients

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="amazon-sp-api-clients",
    version=amazon_sp_api_clients.version,
    author=amazon_sp_api_clients.author,
    author_email=amazon_sp_api_clients.author_email,
    description=amazon_sp_api_clients.description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=amazon_sp_api_clients.url,
    packages=['amazon_sp_api_clients'],
    install_requires=[
        'peewee',
        'cachetools',
        'requests',
        'boto3',
        'pycryptodome',
        'chardet',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class GetOrdersResponse(__BaseDictObject):
    """
    The response schema for the getOrders operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderList = self._get_value(OrderList, "payload")
        else:
            self.payload: OrderList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderResponse(__BaseDictObject):
    """
    The response schema for the getOrder operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Order = self._get_value(Order, "payload")
        else:
            self.payload: Order = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class OrderList(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "pagination" in data:
            self.pagination: Pagination = self._get_value(Pagination, "pagination")
        else:
            self.pagination: Pagination = None
        if "orders" in data:
            self.orders: _List[Order] = [Order(datum) for datum in data["orders"]]
        else:
            self.orders: _List[Order] = []


class Pagination(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "nextToken" in data:
            self.nextToken: str = self._get_value(str, "nextToken")
        else:
            self.nextToken: str = None


class Order(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "orderDetails" in data:
            self.orderDetails: OrderDetails = self._get_value(OrderDetails, "orderDetails")
        else:
            self.orderDetails: OrderDetails = None


class OrderDetails(__BaseDictObject):
    """
    Details of an order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "customerOrderNumber" in data:
            self.customerOrderNumber: str = self._get_value(str, "customerOrderNumber")
        else:
            self.customerOrderNumber: str = None
        if "orderDate" in data:
            self.orderDate: str = self._get_value(str, "orderDate")
        else:
            self.orderDate: str = None
        if "orderStatus" in data:
            self.orderStatus: str = self._get_value(str, "orderStatus")
        else:
            self.orderStatus: str = None
        if "shipmentDetails" in data:
            self.shipmentDetails: ShipmentDetails = self._get_value(ShipmentDetails, "shipmentDetails")
        else:
            self.shipmentDetails: ShipmentDetails = None
        if "taxTotal" in data:
            self.taxTotal: dict = self._get_value(dict, "taxTotal")
        else:
            self.taxTotal: dict = None
        if "sellingParty" in data:
            self.sellingParty: PartyIdentification = self._get_value(PartyIdentification, "sellingParty")
        else:
            self.sellingParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None
        if "shipToParty" in data:
            self.shipToParty: Address = self._get_value(Address, "shipToParty")
        else:
            self.shipToParty: Address = None
        if "billToParty" in data:
            self.billToParty: PartyIdentification = self._get_value(PartyIdentification, "billToParty")
        else:
            self.billToParty: PartyIdentification = None
        if "items" in data:
            self.items: _List[OrderItem] = [OrderItem(datum) for datum in data["items"]]
        else:
            self.items: _List[OrderItem] = []


class PartyIdentification(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "partyId" in data:
            self.partyId: str = self._get_value(str, "partyId")
        else:
            self.partyId: str = None
        if "address" in data:
            self.address: Address = self._get_value(Address, "address")
        else:
            self.address: Address = None
        if "taxInfo" in data:
            self.taxInfo: TaxRegistrationDetails = self._get_value(TaxRegistrationDetails, "taxInfo")
        else:
            self.taxInfo: TaxRegistrationDetails = None


class TaxRegistrationDetails(__BaseDictObject):
    """
    Tax registration details of the entity.
    """

    def __init__(self, data):
        super().__init__(data)
        if "taxRegistrationType" in data:
            self.taxRegistrationType: str = self._get_value(str, "taxRegistrationType")
        else:
            self.taxRegistrationType: str = None
        if "taxRegistrationNumber" in data:
            self.taxRegistrationNumber: str = self._get_value(str, "taxRegistrationNumber")
        else:
            self.taxRegistrationNumber: str = None
        if "taxRegistrationAddress" in data:
            self.taxRegistrationAddress: Address = self._get_value(Address, "taxRegistrationAddress")
        else:
            self.taxRegistrationAddress: Address = None
        if "taxRegistrationMessages" in data:
            self.taxRegistrationMessages: str = self._get_value(str, "taxRegistrationMessages")
        else:
            self.taxRegistrationMessages: str = None


class Address(__BaseDictObject):
    """
    Address of the party.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "attention" in data:
            self.attention: str = self._get_value(str, "attention")
        else:
            self.attention: str = None
        if "addressLine1" in data:
            self.addressLine1: str = self._get_value(str, "addressLine1")
        else:
            self.addressLine1: str = None
        if "addressLine2" in data:
            self.addressLine2: str = self._get_value(str, "addressLine2")
        else:
            self.addressLine2: str = None
        if "addressLine3" in data:
            self.addressLine3: str = self._get_value(str, "addressLine3")
        else:
            self.addressLine3: str = None
        if "city" in data:
            self.city: str = self._get_value(str, "city")
        else:
            self.city: str = None
        if "county" in data:
            self.county: str = self._get_value(str, "county")
        else:
            self.county: str = None
        if "district" in data:
            self.district: str = self._get_value(str, "district")
        else:
            self.district: str = None
        if "stateOrRegion" in data:
            self.stateOrRegion: str = self._get_value(str, "stateOrRegion")
        else:
            self.stateOrRegion: str = None
        if "postalCode" in data:
            self.postalCode: str = self._get_value(str, "postalCode")
        else:
            self.postalCode: str = None
        if "countryCode" in data:
            self.countryCode: str = self._get_value(str, "countryCode")
        else:
            self.countryCode: str = None
        if "phone" in data:
            self.phone: str = self._get_value(str, "phone")
        else:
            self.phone: str = None


class OrderItem(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "itemSequenceNumber" in data:
            self.itemSequenceNumber: str = self._get_value(str, "itemSequenceNumber")
        else:
            self.itemSequenceNumber: str = None
        if "buyerProductIdentifier" in data:
            self.buyerProductIdentifier: str = self._get_value(str, "buyerProductIdentifier")
        else:
            self.buyerProductIdentifier: str = None
        if "vendorProductIdentifier" in data:
            self.vendorProductIdentifier: str = self._get_value(str, "vendorProductIdentifier")
        else:
            self.vendorProductIdentifier: str = None
        if "title" in data:
            self.title: str = self._get_value(str, "title")
        else:
            self.title: str = None
        if "orderedQuantity" in data:
            self.orderedQuantity: ItemQuantity = self._get_value(ItemQuantity, "orderedQuantity")
        else:
            self.orderedQuantity: ItemQuantity = None
        if "scheduledDeliveryShipment" in data:
            self.scheduledDeliveryShipment: ScheduledDeliveryShipment = self._get_value(
                ScheduledDeliveryShipment, "scheduledDeliveryShipment"
            )
        else:
            self.scheduledDeliveryShipment: ScheduledDeliveryShipment = None
        if "giftDetails" in data:
            self.giftDetails: GiftDetails = self._get_value(GiftDetails, "giftDetails")
        else:
            self.giftDetails: GiftDetails = None
        if "netPrice" in data:
            self.netPrice: Money = self._get_value(Money, "netPrice")
        else:
            self.netPrice: Money = None
        if "taxDetails" in data:
            self.taxDetails: dict = self._get_value(dict, "taxDetails")
        else:
            self.taxDetails: dict = None
        if "totalPrice" in data:
            self.totalPrice: Money = self._get_value(Money, "totalPrice")
        else:
            self.totalPrice: Money = None


class Money(__BaseDictObject):
    """
    An amount of money, including units in the form of currency.
    """

    def __init__(self, data):
        super().__init__(data)
        if "currencyCode" in data:
            self.currencyCode: str = self._get_value(str, "currencyCode")
        else:
            self.currencyCode: str = None
        if "amount" in data:
            self.amount: Decimal = self._get_value(Decimal, "amount")
        else:
            self.amount: Decimal = None


class SubmitAcknowledgementResponse(__BaseDictObject):
    """
    The response schema for the submitAcknowledgement operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: TransactionId = self._get_value(TransactionId, "payload")
        else:
            self.payload: TransactionId = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class TransactionId(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None


class SubmitAcknowledgementRequest(__BaseDictObject):
    """
    The request schema for the submitAcknowledgement operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "orderAcknowledgements" in data:
            self.orderAcknowledgements: _List[OrderAcknowledgementItem] = [
                OrderAcknowledgementItem(datum) for datum in data["orderAcknowledgements"]
            ]
        else:
            self.orderAcknowledgements: _List[OrderAcknowledgementItem] = []


class OrderAcknowledgementItem(__BaseDictObject):
    """
    Details of an individual order being acknowledged.
    """

    def __init__(self, data):
        super().__init__(data)
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "vendorOrderNumber" in data:
            self.vendorOrderNumber: str = self._get_value(str, "vendorOrderNumber")
        else:
            self.vendorOrderNumber: str = None
        if "acknowledgementDate" in data:
            self.acknowledgementDate: str = self._get_value(str, "acknowledgementDate")
        else:
            self.acknowledgementDate: str = None
        if "acknowledgementStatus" in data:
            self.acknowledgementStatus: AcknowledgementStatus = self._get_value(
                AcknowledgementStatus, "acknowledgementStatus"
            )
        else:
            self.acknowledgementStatus: AcknowledgementStatus = None
        if "sellingParty" in data:
            self.sellingParty: PartyIdentification = self._get_value(PartyIdentification, "sellingParty")
        else:
            self.sellingParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None
        if "itemAcknowledgements" in data:
            self.itemAcknowledgements: _List[OrderItemAcknowledgement] = [
                OrderItemAcknowledgement(datum) for datum in data["itemAcknowledgements"]
            ]
        else:
            self.itemAcknowledgements: _List[OrderItemAcknowledgement] = []


class OrderItemAcknowledgement(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "itemSequenceNumber" in data:
            self.itemSequenceNumber: str = self._get_value(str, "itemSequenceNumber")
        else:
            self.itemSequenceNumber: str = None
        if "buyerProductIdentifier" in data:
            self.buyerProductIdentifier: str = self._get_value(str, "buyerProductIdentifier")
        else:
            self.buyerProductIdentifier: str = None
        if "vendorProductIdentifier" in data:
            self.vendorProductIdentifier: str = self._get_value(str, "vendorProductIdentifier")
        else:
            self.vendorProductIdentifier: str = None
        if "acknowledgedQuantity" in data:
            self.acknowledgedQuantity: ItemQuantity = self._get_value(ItemQuantity, "acknowledgedQuantity")
        else:
            self.acknowledgedQuantity: ItemQuantity = None


class ItemQuantity(__BaseDictObject):
    """
    Details of quantity ordered.
    """

    def __init__(self, data):
        super().__init__(data)
        if "amount" in data:
            self.amount: int = self._get_value(int, "amount")
        else:
            self.amount: int = None
        if "unitOfMeasure" in data:
            self.unitOfMeasure: str = self._get_value(str, "unitOfMeasure")
        else:
            self.unitOfMeasure: str = None


class TaxDetails(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "taxRate" in data:
            self.taxRate: Decimal = self._get_value(Decimal, "taxRate")
        else:
            self.taxRate: Decimal = None
        if "taxAmount" in data:
            self.taxAmount: Money = self._get_value(Money, "taxAmount")
        else:
            self.taxAmount: Money = None
        if "taxableAmount" in data:
            self.taxableAmount: Money = self._get_value(Money, "taxableAmount")
        else:
            self.taxableAmount: Money = None
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None


class AcknowledgementStatus(__BaseDictObject):
    """
    Status of acknowledgement.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "description" in data:
            self.description: str = self._get_value(str, "description")
        else:
            self.description: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ShipmentDetails(__BaseDictObject):
    """
    Shipment details required for the shipment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "isPriorityShipment" in data:
            self.isPriorityShipment: bool = self._get_value(convert_bool, "isPriorityShipment")
        else:
            self.isPriorityShipment: bool = None
        if "isScheduledDeliveryShipment" in data:
            self.isScheduledDeliveryShipment: bool = self._get_value(convert_bool, "isScheduledDeliveryShipment")
        else:
            self.isScheduledDeliveryShipment: bool = None
        if "isPslipRequired" in data:
            self.isPslipRequired: bool = self._get_value(convert_bool, "isPslipRequired")
        else:
            self.isPslipRequired: bool = None
        if "isGift" in data:
            self.isGift: bool = self._get_value(convert_bool, "isGift")
        else:
            self.isGift: bool = None
        if "shipMethod" in data:
            self.shipMethod: str = self._get_value(str, "shipMethod")
        else:
            self.shipMethod: str = None
        if "shipmentDates" in data:
            self.shipmentDates: ShipmentDates = self._get_value(ShipmentDates, "shipmentDates")
        else:
            self.shipmentDates: ShipmentDates = None
        if "messageToCustomer" in data:
            self.messageToCustomer: str = self._get_value(str, "messageToCustomer")
        else:
            self.messageToCustomer: str = None


class ShipmentDates(__BaseDictObject):
    """
    Shipment dates.
    """

    def __init__(self, data):
        super().__init__(data)
        if "requiredShipDate" in data:
            self.requiredShipDate: str = self._get_value(str, "requiredShipDate")
        else:
            self.requiredShipDate: str = None
        if "promisedDeliveryDate" in data:
            self.promisedDeliveryDate: str = self._get_value(str, "promisedDeliveryDate")
        else:
            self.promisedDeliveryDate: str = None


class ScheduledDeliveryShipment(__BaseDictObject):
    """
    Dates for the scheduled delivery shipments.
    """

    def __init__(self, data):
        super().__init__(data)
        if "scheduledDeliveryServiceType" in data:
            self.scheduledDeliveryServiceType: str = self._get_value(str, "scheduledDeliveryServiceType")
        else:
            self.scheduledDeliveryServiceType: str = None
        if "earliestNominatedDeliveryDate" in data:
            self.earliestNominatedDeliveryDate: str = self._get_value(str, "earliestNominatedDeliveryDate")
        else:
            self.earliestNominatedDeliveryDate: str = None
        if "latestNominatedDeliveryDate" in data:
            self.latestNominatedDeliveryDate: str = self._get_value(str, "latestNominatedDeliveryDate")
        else:
            self.latestNominatedDeliveryDate: str = None


class GiftDetails(__BaseDictObject):
    """
    Gift details for the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "giftMessage" in data:
            self.giftMessage: str = self._get_value(str, "giftMessage")
        else:
            self.giftMessage: str = None
        if "giftWrapId" in data:
            self.giftWrapId: str = self._get_value(str, "giftWrapId")
        else:
            self.giftWrapId: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class TaxLineItem(list, _List["TaxDetails"]):
    """
    A list of tax line items.
    """

    def __init__(self, data):
        super().__init__([TaxDetails(datum) for datum in data])
        self.data = data


class Decimal(str):
    """
    A decimal number with no loss of precision. Useful when precision loss is unacceptable, as with currencies. Follows RFC7159 for number representation.
    """


class VendorDirectFulfillmentOrdersV1Client(__BaseClient):
    def getOrders(
        self,
        createdAfter: str,
        createdBefore: str,
        shipFromPartyId: str = None,
        status: str = None,
        limit: int = None,
        sortOrder: str = None,
        nextToken: str = None,
        includeDetails: str = None,
    ):
        """
                Returns a list of purchase orders created during the time frame that you specify. You define the time frame using the createdAfter and createdBefore parameters. You must use both parameters. You can choose to get only the purchase order numbers by setting the includeDetails parameter to false. In that case, the operation returns a list of purchase order numbers. You can then call the getOrder operation to return the details of a specific order.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/vendor/directFulfillment/orders/v1/purchaseOrders"
        params = {}
        if shipFromPartyId is not None:
            params["shipFromPartyId"] = shipFromPartyId
        if status is not None:
            params["status"] = status
        if limit is not None:
            params["limit"] = limit
        if createdAfter is not None:
            params["createdAfter"] = createdAfter
        if createdBefore is not None:
            params["createdBefore"] = createdBefore
        if sortOrder is not None:
            params["sortOrder"] = sortOrder
        if nextToken is not None:
            params["nextToken"] = nextToken
        if includeDetails is not None:
            params["includeDetails"] = includeDetails
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrdersResponse,
            400: GetOrdersResponse,
            403: GetOrdersResponse,
            404: GetOrdersResponse,
            415: GetOrdersResponse,
            429: GetOrdersResponse,
            500: GetOrdersResponse,
            503: GetOrdersResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrder(
        self,
        purchaseOrderNumber: str,
    ):
        """
                Returns purchase order information for the purchaseOrderNumber that you specify.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/vendor/directFulfillment/orders/v1/purchaseOrders/{purchaseOrderNumber}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderResponse,
            400: GetOrderResponse,
            401: GetOrderResponse,
            403: GetOrderResponse,
            404: GetOrderResponse,
            415: GetOrderResponse,
            429: GetOrderResponse,
            500: GetOrderResponse,
            503: GetOrderResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def submitAcknowledgement(
        self,
        data: SubmitAcknowledgementRequest,
    ):
        """
                Submits acknowledgements for one or more purchase orders.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/vendor/directFulfillment/orders/v1/acknowledgements"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: SubmitAcknowledgementResponse,
            400: SubmitAcknowledgementResponse,
            403: SubmitAcknowledgementResponse,
            404: SubmitAcknowledgementResponse,
            413: SubmitAcknowledgementResponse,
            415: SubmitAcknowledgementResponse,
            429: SubmitAcknowledgementResponse,
            500: SubmitAcknowledgementResponse,
            503: SubmitAcknowledgementResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class GetAuthorizationCodeResponse(__BaseDictObject):
    """
    The response schema for the GetAuthorizationCode operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: AuthorizationCode = self._get_value(AuthorizationCode, "payload")
        else:
            self.payload: AuthorizationCode = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class AuthorizationCode(__BaseDictObject):
    """
    A Login with Amazon (LWA) authorization code.
    """

    def __init__(self, data):
        super().__init__(data)
        if "authorizationCode" in data:
            self.authorizationCode: str = self._get_value(str, "authorizationCode")
        else:
            self.authorizationCode: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class AuthorizationV1Client(__BaseClient):
    def getAuthorizationCode(
        self,
        sellingPartnerId: str,
        developerId: str,
        mwsAuthToken: str,
    ):
        """
                With the getAuthorizationCode operation, you can request a Login With Amazon (LWA) authorization code that will allow you to call a Selling Partner API on behalf of a seller who has already authorized you to call Amazon Marketplace Web Service (Amazon MWS). You specify a developer ID, an MWS auth token, and a seller ID. Taken together, these represent the Amazon MWS authorization that the seller previously granted you. The operation returns an LWA authorization code that can be exchanged for a refresh token and access token representing authorization to call the Selling Partner API on the seller's behalf. By using this API, sellers who have already authorized you for Amazon MWS do not need to re-authorize you for the Selling Partner API.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/authorization/v1/authorizationCode"
        params = {}
        if sellingPartnerId is not None:
            params["sellingPartnerId"] = sellingPartnerId
        if developerId is not None:
            params["developerId"] = developerId
        if mwsAuthToken is not None:
            params["mwsAuthToken"] = mwsAuthToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetAuthorizationCodeResponse,
            400: GetAuthorizationCodeResponse,
            403: GetAuthorizationCodeResponse,
            404: GetAuthorizationCodeResponse,
            413: GetAuthorizationCodeResponse,
            415: GetAuthorizationCodeResponse,
            429: GetAuthorizationCodeResponse,
            500: GetAuthorizationCodeResponse,
            503: GetAuthorizationCodeResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class Error(__BaseDictObject):
    """
    An error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class CreateFeedResponse(__BaseDictObject):
    """
    Response schema.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feedId" in data:
            self.feedId: str = self._get_value(str, "feedId")
        else:
            self.feedId: str = None


class Feed(__BaseDictObject):
    """
    Detailed information about the feed.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feedId" in data:
            self.feedId: str = self._get_value(str, "feedId")
        else:
            self.feedId: str = None
        if "feedType" in data:
            self.feedType: str = self._get_value(str, "feedType")
        else:
            self.feedType: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "createdTime" in data:
            self.createdTime: str = self._get_value(str, "createdTime")
        else:
            self.createdTime: str = None
        if "processingStatus" in data:
            self.processingStatus: str = self._get_value(str, "processingStatus")
        else:
            self.processingStatus: str = None
        if "processingStartTime" in data:
            self.processingStartTime: str = self._get_value(str, "processingStartTime")
        else:
            self.processingStartTime: str = None
        if "processingEndTime" in data:
            self.processingEndTime: str = self._get_value(str, "processingEndTime")
        else:
            self.processingEndTime: str = None
        if "resultFeedDocumentId" in data:
            self.resultFeedDocumentId: str = self._get_value(str, "resultFeedDocumentId")
        else:
            self.resultFeedDocumentId: str = None


class GetFeedsResponse(__BaseDictObject):
    """
    Response schema.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feeds" in data:
            self.feeds: FeedList = self._get_value(FeedList, "feeds")
        else:
            self.feeds: FeedList = None
        if "nextToken" in data:
            self.nextToken: str = self._get_value(str, "nextToken")
        else:
            self.nextToken: str = None


class FeedDocument(__BaseDictObject):
    """
    Information required for the feed document.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feedDocumentId" in data:
            self.feedDocumentId: str = self._get_value(str, "feedDocumentId")
        else:
            self.feedDocumentId: str = None
        if "url" in data:
            self.url: str = self._get_value(str, "url")
        else:
            self.url: str = None
        if "compressionAlgorithm" in data:
            self.compressionAlgorithm: str = self._get_value(str, "compressionAlgorithm")
        else:
            self.compressionAlgorithm: str = None


class FeedOptions(__BaseDictObject):
    """
    Additional options to control the feed. These vary by feed type.
    """

    def __init__(self, data):
        super().__init__(data)


class CreateFeedSpecification(__BaseDictObject):
    """
    Information required to create the feed.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feedType" in data:
            self.feedType: str = self._get_value(str, "feedType")
        else:
            self.feedType: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "inputFeedDocumentId" in data:
            self.inputFeedDocumentId: str = self._get_value(str, "inputFeedDocumentId")
        else:
            self.inputFeedDocumentId: str = None
        if "feedOptions" in data:
            self.feedOptions: FeedOptions = self._get_value(FeedOptions, "feedOptions")
        else:
            self.feedOptions: FeedOptions = None


class CreateFeedDocumentSpecification(__BaseDictObject):
    """
    Specifies the content type for the createFeedDocument operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "contentType" in data:
            self.contentType: str = self._get_value(str, "contentType")
        else:
            self.contentType: str = None


class CreateFeedDocumentResponse(__BaseDictObject):
    """
    Information required to upload a feed document's contents.
    """

    def __init__(self, data):
        super().__init__(data)
        if "feedDocumentId" in data:
            self.feedDocumentId: str = self._get_value(str, "feedDocumentId")
        else:
            self.feedDocumentId: str = None
        if "url" in data:
            self.url: str = self._get_value(str, "url")
        else:
            self.url: str = None


class FeedList(list, _List["Feed"]):
    """
    A list of feeds.
    """

    def __init__(self, data):
        super().__init__([Feed(datum) for datum in data])
        self.data = data


class Feeds20210630Client(__BaseClient):
    def getFeeds(
        self,
        feedTypes: _List[str] = None,
        marketplaceIds: _List[str] = None,
        pageSize: int = None,
        processingStatuses: _List[str] = None,
        createdSince: str = None,
        createdUntil: str = None,
        nextToken: str = None,
    ):
        """
                Returns feed details for the feeds that match the filters that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/feeds"
        params = {}
        if feedTypes is not None:
            params["feedTypes"] = ",".join(map(str, feedTypes))
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if pageSize is not None:
            params["pageSize"] = pageSize
        if processingStatuses is not None:
            params["processingStatuses"] = ",".join(map(str, processingStatuses))
        if createdSince is not None:
            params["createdSince"] = createdSince
        if createdUntil is not None:
            params["createdUntil"] = createdUntil
        if nextToken is not None:
            params["nextToken"] = nextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetFeedsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createFeed(
        self,
        data: CreateFeedSpecification,
    ):
        """
                Creates a feed. Upload the contents of the feed document before calling this operation.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0083 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/feeds"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: CreateFeedResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getFeed(
        self,
        feedId: str,
    ):
        """
                Returns feed details (including the resultDocumentId, if available) for the feed that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 2.0 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/feeds/{feedId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: Feed,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def cancelFeed(
        self,
        feedId: str,
    ):
        """
                Cancels the feed that you specify. Only feeds with processingStatus=IN_QUEUE can be cancelled. Cancelled feeds are returned in subsequent calls to the getFeed and getFeeds operations.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/feeds/{feedId}"
        params = {}
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: None,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createFeedDocument(
        self,
        data: CreateFeedDocumentSpecification,
    ):
        """
                Creates a feed document for the feed type that you specify. This operation returns a presigned URL for uploading the feed document contents. It also returns a feedDocumentId value that you can pass in with a subsequent call to the createFeed operation.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0083 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/documents"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            201: CreateFeedDocumentResponse,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getFeedDocument(
        self,
        feedDocumentId: str,
    ):
        """
                Returns the information required for retrieving a feed document's contents.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/feeds/2021-06-30/documents/{feedDocumentId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: FeedDocument,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))
